import platform
import re
from bloonix.runcmd import RunCmd

####################
# Linux
def get_linux_distribution():
    d = {}
    try:
        r = RunCmd(command=["lsb_release", "-a"])
        for line in r.stdout.splitlines():
            m = re.search(r"^(.+?):\s*(.+)", line)
            if m:
                d[m.group(1)] = m.group(2)
    except Exception: pass
    return d

def get_linux_meminfo(key):
    size = 0
    try:
        meminfo = get_file_content("/proc/meminfo")
        for line in meminfo.splitlines():
            m = re.search(r"%s:\s*(\d+)\s+(..)" % key, line)
            if m:
                size = int(m.group(1))
                unit = m.group(2)
                if unit == "kB":
                    size = size * 1024
                break
    except Exception: pass
    return size

def get_linux_processors():
    num = 0
    try:
        cpuinfo = get_file_content("/proc/cpuinfo")
        for line in cpuinfo.splitlines():
            m = re.search(r"^(processor|# processors)\s*:\s*(\d+)", line)
            if m:
                if m.group(1) == "processor":       # x86, arm
                    num += 1
                elif m.group(1) == "# processors":  # s390
                    num = int(m.group(2))
                    break
    except Exception: pass
    return num

def get_linux_interfaces():
    d = []
    try:
        netdev = get_file_content("/proc/net/dev")
        for line in netdev.splitlines():
            m = re.search(r"^\s*(\w+):", line)
            if m:
                d.append(m.group(1))
    except Exception: pass
    return d

def get_linux_disks():
    d = []
    try:
        r = RunCmd(command=["lsblk", "-d"])
        for line in r.stdout.splitlines():
            parts = re.split(r"\s+", line)
            if len(parts) > 4 and parts[5] == "disk":
                d.append(parts[0])
    except Exception: pass
    return d

def get_linux_facts():
    return {
        "Distrubution": get_linux_distribution(),
        "Memory": get_linux_meminfo("MemTotal"),
        "Swap": get_linux_meminfo("SwapTotal"),
        "Processors": get_linux_processors(),
        "Interfaces": get_linux_interfaces(),
        "Disks": get_linux_disks(),
    }

####################
# Main
def get_file_content(file):
    f = open(file, "r")
    c = f.read()
    f.close()
    return c

def get_os_specific_facts():
    if platform.system() == "Linux":
        return get_linux_facts()

def get_facts():
    info = {
        "System": platform.system(),
        "Hostname": platform.node(),
        "Release": platform.release(),
        "Version": platform.version(),
        "Machine": platform.machine(),
    }
    info.update(get_os_specific_facts())
    return info
