import re
from bloonix.config import parse_file
from bloonix.validator import validate

class Validate:
    def main_config(config_file):
        if config_file is not None:
            config = parse_file(config_file)
        else:
            config = {}

        config = validate(config, {
            "worker": {
                "type": int,
                "minvalue": 1,
                "default": 4
            },
            "max_concurrent_hosts": {
                "type": int,
                "minvalue": 0,
                "default": 0
            },
            "max_concurrent_checks": {
                "type": int,
                "minvalue": 0,
                "default": 0
            },
            "user": {
                "type": str,
                "default": "bloonix"
            },
            "group": {
                "type": str,
                "default": "bloonix"
            },
            "plugin_path": {
                "type": str,
                "default": (
                    "/usr/lib/bloonix/plugins,"
                    "/usr/local/lib/bloonix/plugins")
            },
            "simple_plugin_path": {
                "type": str,
                "default": (
                    "/usr/lib/bloonix/simple-plugins"
                    "/usr/local/lib/bloonix/simple-plugins")
            },
            "plugin_cache_dir": {
                "type": str,
                "default": "/var/lib/bloonix/plugins"
            },
            "server": {
                "type": dict
            },
            "logger": {
                "type": dict,
                "allow_empty": True
            },
            "host": {
                "type": dict,
                "allow_empty": True,
                "optional": True,
            },
            "location_group": {
                "type": dict,
                "optional": True
            },
            "testmode": {
                "type": bool,
                "default": False
            },
            "environ": {
                "type": dict,
                "allow_empty": True,
                "default": {}
            },
        })

        if config["max_concurrent_checks"] == 0:
            config["max_concurrent_checks"] = config["worker"]

        if config["max_concurrent_hosts"] == 0:
            if config["max_concurrent_checks"] > 1:
                config["max_concurrent_hosts"] = int(
                    config["max_concurrent_checks"] / 2)
            else:
                config["max_concurrent_hosts"] == 1

        for key in ["plugin_path", "simple_plugin_path"]:
            value = config[key]
            value = re.sub(r"[\s\n]", "", value)
            value = re.split(",", value)
            config[key] = value

        if "host" in config and config["host"] is not None:
            config["host"] = Validate.host_config(config["host"])

        if config["location_group"] is not None:
            config["location_group"] = Validate.location_group(
                config["location_group"])

            if config["location_group"]["location_id"] == "0":
                config["location_group"] = None

        return config

    def host_config(config):
        return validate(config, {
            "host_id": {
                "type": str
            },
            "password": {
                "type": str
            },
            "proxy": {
                "type": str,
                "optional": True
            },
        })

    def location_group(config):
        return validate(config, {
            "location_id": {
                "type": str
            },
            "password": {
                "type": str
            },
            "proxy": {
                "type": str,
                "optional": True
            },
        })

    def server_response(response, datatype):
        if not isinstance(response, dict):
            raise Exception("server response is not a dict")
        if response.get("status", None) not in ["ok", "err"]:
            raise Exception("invalid status in response")
        if "message" in response and not isinstance(response["message"], str):
            raise Exception("server message is not a string")

        if "data" not in response:
            return

        if datatype == "services":
            if not isinstance(response["data"], dict):
                raise Exception("data is not a dict")
            data = response["data"]
            if "services" not in data:
                raise Exception("no services in response")
            if ("poll_interval" in data
                    and not isinstance(data["poll_interval"], int)):
                raise Exception("invalid poll interval")
            if not isinstance(response["data"]["services"], list):
                raise Exception("services is not a list")
        elif datatype == "location_group":
            if not isinstance(response["data"], list):
                raise Exception("data is not a list")
            for row in response["data"]:
                if ("host_id" not in row
                        or "password" not in row
                        or not isinstance(row["host_id"], int)
                        or not isinstance(row["password"], str)):
                    raise Exception("location groups: invalid data format")
